@@ -1,9 +1,11 @@
 from text_explainability.global_explanation import (TokenFrequency, TokenInformation,
                                                     KMedoids, LabelwiseKMedoids, MMDCritic, LabelwiseMMDCritic)
-from text_explainability.local_explanation import LIME, KernelSHAP, Anchor, LocalTree
+from text_explainability.local_explanation import LIME, KernelSHAP, Anchor, LocalTree, LocalRules
 from text_explainability.utils import (default_tokenizer, default_detokenizer,
                                        word_tokenizer, word_detokenizer,
                                        character_tokenizer, character_detokenizer)
+from text_explainability.data import from_string, import_data, train_test_split
+from text_explainability.model import from_sklearn
 
 
-__version__ = '0.5.0'
+__version__ = '0.5.1'
