@@ -1,6 +1,7 @@
 """Sample an (informative) subset from the data.
 
 Todo:
+
     * Sample (informative?) subset from data
     * Refactor to make sampling base class
     * Add ability to perform MMD critic on a subset (e.g. single class)
@@ -135,13 +136,18 @@ class MMDCritic(PrototypeSampler):
         Args:
             n (int, optional): Number of prototypes to select. Defaults to 5.
 
+        Raises:
+            ValueError: Cannot select more instances than the total number of instances.
+
         Returns:
             Sequence[MemoryTextInstance]: List of prototype instances.
 
         .. _MMD-critic implementation:
             https://github.com/maxidl/MMD-critic/blob/main/mmd_critic.py
         """
-        assert n <= len(self.instances), f'Cannot select more than all instances ({len(self.instances)}.'
+        if n > len(self.instances):
+            raise ValueError(f'Cannot select more than all instances ({len(self.instances)}.')
+
         K = self.K
         colsum = self.colsum.copy() * 2
         sample_indices = np.array(list(self.instances))
@@ -179,6 +185,7 @@ class MMDCritic(PrototypeSampler):
 
         Raises:
             Exception: `MMDCritic.prototypes()` must first be run before being able to determine the criticisms.
+            ValueError: Unknown regularizer or requested more criticisms than there are samples left.
 
         Returns:
             Sequence[MemoryTextInstance]: List of criticism instances.
@@ -189,10 +196,12 @@ class MMDCritic(PrototypeSampler):
         if self._prototypes is None:
             raise Exception('Calculating criticisms requires prototypes. Run `MMDCritic.prototypes()` first.')
         regularizers = {None, 'logdet', 'iterative'}
-        assert regularizer in regularizers, \
-            f'Unknown regularizer "{regularizer}", choose from {regularizers}.'
-        assert n <= (len(self.instances) - len(self._prototypes)), \
-            f'Cannot select more than instances excluding prototypes ({len(self.instances) - len(self._prototypes)})'
+
+        if regularizer not in regularizers:
+            raise ValueError(f'Unknown {regularizer=}. Choose from {regularizers}.')
+        if n > (len(self.instances) - len(self._prototypes)):
+            raise ValueError('Cannot select more than instances excluding prototypes ',
+                             f'({len(self.instances) - len(self._prototypes)})')
 
         prototypes = np.array([p.identifier for p in self._prototypes])
 
@@ -215,7 +224,7 @@ class MMDCritic(PrototypeSampler):
             s1 = np.abs(s1)
 
             if regularizer == 'logdet':
-                diag = np.diagonal(K)[candidate_indices]
+                diag = np.diagonal(K + 1)[candidate_indices]  # TODO: look at hotfix
                 if inverse_of_prev_selected is not None:
                     temp = K[selected, :][:, candidate_indices]
                     temp2 = np.dot(inverse_of_prev_selected, temp) 
