@@ -1,7 +1,9 @@
 """Local explanations explain why a model made a prediction for a single instance.
 
 Todo:
+
     * Implement Anchors
+    * Various bugfixes
 """
 
 import numpy as np
@@ -16,7 +18,7 @@ from sklearn.linear_model import Ridge
 from sklearn.tree import DecisionTreeClassifier
 
 from text_explainability.data.augmentation import (LocalTokenPertubator,
-                                                   TokenReplacement)
+                                                   LeaveOut)
 from text_explainability.data.weights import (exponential_kernel,
                                               pairwise_distances)
 from text_explainability.default import Readable
@@ -73,7 +75,7 @@ class LocalExplanation(Readable):
         super().__init__()
         self.env = default_env(env)
         if augmenter is None:
-            augmenter = TokenReplacement(env=self.env, detokenizer=default_detokenizer)
+            augmenter = LeaveOut(env=self.env, detokenizer=default_detokenizer)
         if isinstance(labelset, LabelProvider) and hasattr(labelset, 'labelset'):
             labelset = list(labelset.labelset)
         elif labelset is None and self.env is not None:
@@ -141,7 +143,10 @@ class LocalExplanation(Readable):
 
         if predict:
             return provider, perturbed, y
-        return provider, perturbed        
+        return provider, perturbed
+
+    def explain(self, *args, **kwargs):
+        return self(*args, **kwargs)    
 
 
 class WeightedExplanation:
@@ -166,7 +171,7 @@ class WeightedExplanation:
 
 class LIME(LocalExplanation, WeightedExplanation):
     def __init__(self,
-                 env: Optional[AbstractEnvironment],
+                 env: Optional[AbstractEnvironment] = None,
                  local_model: Optional[LinearSurrogate] = None,
                  kernel: Optional[Callable] = None,
                  kernel_width: Union[int, float] = 25,
@@ -225,6 +230,9 @@ class LIME(LocalExplanation, WeightedExplanation):
                 instance. Defaults to True.
             distance_metric (str, optional): Distance metric for local weighting. Defaults to 'cosine'.
 
+        Raises:
+            ValueError: Can only provide labels from labelset if self.labelset is not None
+
         Returns:
             FeatureAttribution: [description]
 
@@ -237,7 +245,8 @@ class LIME(LocalExplanation, WeightedExplanation):
 
             n_labels = sum(1 for _ in iter(labels))
             if n_labels > 0 and isinstance(next(iter(labels)), str):
-                assert self.labelset is not None, 'can only provide label names when such a list exists'
+                if self.labelset is None:
+                    raise ValueError('Can only provide label names when such a list exists in self.labelset')
                 labels = [self.labelset.index(label) for label in labels]
 
         # Generate neighborhood samples
@@ -371,7 +380,7 @@ class KernelSHAP(LocalExplanation):
                                                      contiguous=False, n_samples=n_samples,
                                                      add_background_instance=True)
 
-        # To-do: exclude non-varying feature groups
+        # TODO: exclude non-varying feature groups
         y_null, y = y[-1], y[1:-1]
         y -= y_null
         used_features = np.arange(perturbed.shape[1])
@@ -385,6 +394,7 @@ class KernelSHAP(LocalExplanation):
             M = perturbed.shape[1]
             Z = np.sum(perturbed[1:-1], axis=1).astype(int)
             weight_vector = np.array([(M - 1) / (math.comb(M, m) * m * (M - m)) for m in range(1, M)])
+            weight_vector = np.append(weight_vector, [np.mean(weight_vector)])  # TODO: replace hotfix
             weight_vector /= np.sum(weight_vector)
             kernel_weights = weight_vector[Z - 1]
 
@@ -410,7 +420,7 @@ class KernelSHAP(LocalExplanation):
 
 class Anchor(LocalExplanation):
     def __init__(self,
-                 env: Optional[AbstractEnvironment],
+                 env: Optional[AbstractEnvironment] = None,
                  labelset: Optional[Union[Sequence[str], LabelProvider]] = None,
                  augmenter: Optional[LocalTokenPertubator] = None,
                  seed: int = 0):
@@ -447,11 +457,17 @@ class Anchor(LocalExplanation):
                     epsilon: float = 0.1,
                     max_anchor_size: Optional[int] = None,
                     batch_size: int = 20):
-        assert beam_size >= 1, f'beam size should be at least 1, but is {beam_size}'
-        assert 0.0 <= min_confidence <= 0.95, f'min_confidence should be a value in [0, 1], but is {min_confidence}'
-        assert 0.0 <= delta <= 0.95, f'delta should be a value in [0, 1], but is {delta}'
-        assert 0.0 <= epsilon <= 0.95, f'epsilon should be a value in [0, 1], but is {epsilon}'
-        assert batch_size > 2, 'requires positive batch size'
+        # TODO: add value checking to decorator
+        if beam_size < 1:
+            raise ValueError(f'{beam_size=} should be at least 1.')
+        if not (0.0 <= min_confidence <= 1.0):
+            raise ValueError(f'{min_confidence=} should be in range [0, 1].')
+        if not (0.0 <= delta <= 1.0):
+            raise ValueError(f'{delta=} should be in range [0, 1].')
+        if not (0.0 <= epsilon <= 1.0):
+            raise ValueError(f'{epsilon=} should be in range [0, 1].')
+        if batch_size < 2:
+            raise ValueError(f'{batch_size=} should be at least 2.')
 
         for batch in provider.instance_chunker(batch_size):
             y = list(model.predict_proba_raw(batch))[-1][-1]  # todo: only look at probs of one class
